package com.jforeach.mazegame;

import java.util.ArrayList;

import com.jforeach.mazegame.R;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.view.MotionEvent;
import android.view.LayoutInflater;
import android.view.View;

public class GameView extends View {
	
	//width and height of the whole maze and width of lines which
	//make the walls
	private int width, height, lineWidth;
	//size of the maze i.e. number of cells in it
	private int mazeSizeX, mazeSizeY;
	//width and height of cells in the maze
	float cellWidth, cellHeight;
	//the following store result of cellWidth+lineWidth 
	//and cellHeight+lineWidth respectively 
	float totalCellWidth, totalCellHeight;
	//the finishing point of the maze
	private int mazeFinishX, mazeFinishY;
	private Maze maze;
	private Activity context;
	private Paint line, red, background;
	private int getTotalCellWidth;
	private Bitmap cluster_standing = BitmapFactory.decodeResource(getResources(), R.drawable.cluster_standing);
	private Bitmap enklaavi_standing = BitmapFactory.decodeResource(getResources(), R.drawable.enklaavi_standing);
	private Bitmap satky_standing = BitmapFactory.decodeResource(getResources(), R.drawable.satky_standing);
	private Bitmap cluster_drinking = BitmapFactory.decodeResource(getResources(), R.drawable.cluster_drinking);
	private Bitmap enklaavi_drinking = BitmapFactory.decodeResource(getResources(), R.drawable.enklaavi_drinking);
	private Bitmap satky_drinking = BitmapFactory.decodeResource(getResources(), R.drawable.satky_drinking);
	private Bitmap cluster_passed_out = BitmapFactory.decodeResource(getResources(), R.drawable.cluster_passed_out);
	private Bitmap beer = BitmapFactory.decodeResource(getResources(), R.drawable.beer);
	private Bitmap kebab = BitmapFactory.decodeResource(getResources(), R.drawable.kebab);
	ArrayList<Item> beerlist = new ArrayList();
	
	private boolean finishGame = false;
	private boolean passOutDialogShown = false;
	
	Player player = Player.getInstance();
	NonPlayer npc1 = new NonPlayer(2);
	NonPlayer npc2 = new NonPlayer(3);
//	Beer b1 = new Beer(beer, 0, 2);
	


	
	public GameView(Context context, Maze maze) {
		super(context);
		this.context = (Activity)context;
		this.maze = maze;
		mazeFinishX = maze.getFinalX();
		mazeFinishY = maze.getFinalY();
		mazeSizeX = maze.getMazeWidth();
		mazeSizeY = maze.getMazeHeight();
		
		
		
		line = new Paint();
		line.setColor(getResources().getColor(R.color.line));
		red = new Paint();
		red.setColor(getResources().getColor(R.color.position));
		background = new Paint();
		background.setColor(getResources().getColor(R.color.game_bg));
		setFocusable(true);
		this.setFocusableInTouchMode(true);
		
		for (int i = 1; i < 6; i=i+2) {
			for (int j = 1; j < 7; j=j+2) {
				if (i == 5) beerlist.add(new Kebab(kebab, j, i));
				else beerlist.add(new Beer(beer, j, i));
			}
			
		}
		
		
	}
	protected void onSizeChanged(int w, int h, int oldw, int oldh) {
		width = (w < h)?w:h;
		height = width;         //for now square mazes
		lineWidth = 1;          //for now 1 pixel wide walls
		cellWidth = (width - ((float)mazeSizeX*lineWidth)) / mazeSizeX;
		totalCellWidth = cellWidth+lineWidth;
		cellHeight = (height - ((float)mazeSizeY*lineWidth)) / mazeSizeY;
		totalCellHeight = cellHeight+lineWidth;
		red.setTextSize(cellHeight*0.75f);
		super.onSizeChanged(w, h, oldw, oldh);
	}
	
	protected void onDraw(Canvas canvas) {
		//fill in the background
		canvas.drawRect(0, 0, width, height, background);
		
		boolean[][] hLines = maze.getHorizontalLines();
		boolean[][] vLines = maze.getVerticalLines();
		//iterate over the boolean arrays to draw walls
		for(int i = 0; i < mazeSizeX; i++) {
			for(int j = 0; j < mazeSizeY; j++){
				float x = j * totalCellWidth;
				float y = i * totalCellHeight;
				if(j < mazeSizeX - 1 && vLines[i][j]) {
					//we'll draw a vertical line
					canvas.drawLine(x + cellWidth,   //start X
									y,               //start Y
									x + cellWidth,   //stop X
									y + cellHeight,  //stop Y
									line);
				}
				if(i < mazeSizeY - 1 && hLines[i][j]) {
					//we'll draw a horizontal line
					canvas.drawLine(x,               //startX 
									y + cellHeight,  //startY 
								    x + cellWidth,   //stopX 
								    y + cellHeight,  //stopY 
									line);
				}
			}
		}

		//draw the player		
		player.SetBitmapW(cluster_standing);
		player.SetBitmapD(cluster_drinking);
		player.setBitmapP(cluster_passed_out);
		
		//draw le non player
		
		npc1.SetBitmapW(enklaavi_standing);
		npc2.SetBitmapW(satky_standing);
		npc1.SetBitmapD(enklaavi_drinking);
		npc2.SetBitmapD(satky_drinking);

		
		
		npc1.Draw(canvas, maze, this);
		npc2.Draw(canvas, maze, this);
		player.Draw(canvas, maze, this);
		
		for (int i = 0; i < beerlist.size(); i++) {
			beerlist.get(i).Draw(canvas, this);
		}
		
		
//		b1.Draw(canvas, this);

		//draw the finishing point indicator
//		canvas.drawText("F",
//						(mazeFinishX * totalCellWidth)+(cellWidth*0.25f),
//						(mazeFinishY * totalCellHeight)+(cellHeight*0.75f),
//						red);
	}
//	@Override
//	public boolean onKeyDown(int keyCode, KeyEvent evt) {
//		boolean moved = false;
//		switch(keyCode) {
//			case KeyEvent.KEYCODE_DPAD_UP:
//				moved = maze.move(Maze.UP);
//				break;
//			case KeyEvent.KEYCODE_DPAD_DOWN:
//				moved = maze.move(Maze.DOWN);
//				break;
//			case KeyEvent.KEYCODE_DPAD_RIGHT:
//				moved = maze.move(Maze.RIGHT);
//				break;
//			case KeyEvent.KEYCODE_DPAD_LEFT:
//				moved = maze.move(Maze.LEFT);
//				break;
//			default:
//				return super.onKeyDown(keyCode,evt);
//		}
//		if(moved) {
//			//the ball was moved so we'll redraw the view
//			invalidate();
//			if(maze.isGameComplete()) {
//				AlertDialog.Builder builder = new AlertDialog.Builder(context);
//				builder.setTitle(context.getText(R.string.finished_title));
//				LayoutInflater inflater = context.getLayoutInflater();
//				View view = inflater.inflate(R.layout.finish, null);
//				builder.setView(view);
//				View closeButton =view.findViewById(R.id.closeGame);
//				closeButton.setOnClickListener(new OnClickListener() {
//					@Override
//					public void onClick(View clicked) {
//						if(clicked.getId() == R.id.closeGame) {
//							context.finish();
//						}
//					}
//				});
//				AlertDialog finishDialog = builder.create();
//				finishDialog.show();
//			}
//		}
//		return true;
//	}
	
	@Override
	public boolean onTouchEvent(MotionEvent event) {
		boolean moved = false;
		boolean player_consumed = false;
		int x,y;
		if (event.getAction() == MotionEvent.ACTION_DOWN && !player.getPassedOut()) {
			
			x = (int)Math.round(event.getX());
			y = (int)Math.round(event.getY());
			
			player.MovePlayer(x, y, getWidth(), getHeight(), maze);
			
			if (player.getDirty()) {
				
					player.setDirty();
					npc1.MoveNonPlayer(1, 0, maze, player);
					npc2.MoveNonPlayer(1, 0, maze, player);
					
					player.setDrinking(false);
					npc1.setDrinking(false);
					npc2.setDrinking(false);
					
					for (int i = 0; i < beerlist.size(); i++) {
						
						if (CompareCoordinates(player, beerlist.get(i)) != null && beerlist.get(i).getClass().equals(Beer.class) ) {
							beerlist.remove(i);
							player.setBloodAlcohol(player.getBloodAlcohol() + 0.75);
							player_consumed = true;
							player.setDrinking(true);
						}
						
						else if (CompareCoordinates(player, beerlist.get(i)) != null && beerlist.get(i).getClass().equals(Kebab.class)) {
							beerlist.remove(i);
							player.setBloodAlcohol(player.getBloodAlcohol() * 0.5);
							player_consumed = true;
						}

						
					}
					
					
					for (int i = 0; i < beerlist.size(); i++) {
						
						if (CompareCoordinates(npc1, beerlist.get(i)) != null) {
							beerlist.remove(i);
							npc1.setDrinking(true);
						}
					}
					for (int i = 0; i < beerlist.size(); i++) {
						
						if (CompareCoordinates(npc2, beerlist.get(i)) != null) {
							beerlist.remove(i);
							npc2.setDrinking(true);
						}
					}
					

					if (!player_consumed && player.getBloodAlcohol() > 0.5) player.setBloodAlcohol(player.getBloodAlcohol() - 0.05);
					if (player.getBloodAlcohol() > 2.0) {
						finishGame = true;
					}
				}
			
			
		}
		
		
//		if(moved) {
//		//the ball was moved so we'll redraw the view
		
		
		invalidate();
		if (beerlist.isEmpty() && !finishGame) {
			alert(this);
			finishGame=true;
		}
		if (finishGame && !passOutDialogShown) { // == pass out
		
			player.setPassedOut();
			invalidate();
			PassOut(this);
			passOutDialogShown = true;
			
		}
		
		
//		if(maze.isGameComplete()) {
//			AlertDialog.Builder builder = new AlertDialog.Builder(context);
//			builder.setTitle(context.getText(R.string.finished_title));
//			LayoutInflater inflater = context.getLayoutInflater();
//			View view = inflater.inflate(R.layout.finish, null);
//			builder.setView(view);
//			
//			View closeButton =view.findViewById(R.id.closeGame);
//			
//			closeButton.setOnClickListener(new OnClickListener() {
//				@Override
//				public void onClick(View clicked) {
//					if(clicked.getId() == R.id.closeGame) {
//						context.finish();
//					}
//				}
//			});
//			AlertDialog finishDialog = builder.create();
//			finishDialog.show();
//		}
//	}
	
		return true;
	}
	
public float getTotalCellHeight () {
	return totalCellHeight;
}

public float getTotalCellWidth () {
	return totalCellWidth;
}

public float getCellWidth() {
	return cellWidth;
}

public float getCellHeight() {
	return cellHeight;
}

public Paint getRed() {
	return red;
}

public Item CompareCoordinates(Character c, Item i) {

	if (c.getCurrentX() == i.getX() && c.getCurrentY()==i.getY()) return i;
	else return null;
}

public void Invalidate() {
	invalidate();
}

public void alert(View v) {
	AlertDialog.Builder alertDialogBuilder = new AlertDialog.Builder(
			context);

		// set title
		alertDialogBuilder.setTitle("Game Finished!");

		// set dialog message
		alertDialogBuilder
			.setMessage("You might have won!")
			.setCancelable(false)
			.setPositiveButton("Yes",new DialogInterface.OnClickListener() {
				public void onClick(DialogInterface dialog,int id) {
					// if this button is clicked, close
					// current activity
					//MainActivity.this.finish();
					context.finish();
				}
			  })
			.setNegativeButton("No",new DialogInterface.OnClickListener() {
				public void onClick(DialogInterface dialog,int id) {
					// if this button is clicked, just close
					// the dialog box and do nothing
					dialog.cancel();
				}
			});

			// create alert dialog
			AlertDialog alertDialog = alertDialogBuilder.create();

			// show it
			alertDialog.show();
}

public void PassOut(View v) {
	AlertDialog.Builder alertDialogBuilder = new AlertDialog.Builder(
			context);

		// set title
		alertDialogBuilder.setTitle("Game Finished!");

		// set dialog message
		alertDialogBuilder
			.setMessage("You passed out!")
			.setCancelable(false)
//			.setPositiveButton("Ok",new DialogInterface.OnClickListener() {
//				public void onClick(DialogInterface dialog,int id) {
//					// if this button is clicked, close
//					// current activity
//					//MainActivity.this.finish();
//					context.finish();
//				}
//			  })
			.setNegativeButton("Ok",new DialogInterface.OnClickListener() {
				public void onClick(DialogInterface dialog,int id) {
					// if this button is clicked, just close
					// the dialog box and do nothing
					dialog.cancel();
				}
			});

			// create alert dialog
			AlertDialog alertDialog = alertDialogBuilder.create();

			// show it
			alertDialog.show();
}


}
